from ..Qt import QtWidgets, QtCore
from .._py.tnmh_dialog import Ui_Dialog
from ..lib.pg_objects import PlotItem, RegionItem

from nmreval.data import DSC


class TgCalculator(QtWidgets.QDialog, Ui_Dialog):
    def __init__(self, management, parent=None):
        super().__init__(parent=parent)

        self.setupUi(self)
        self._management = management

        self._plots = {}
        self._tg_baselines = {}
        self.dsc = []

        self.limits = RegionItem(), RegionItem()
        for lim in self.limits:
            self.graphicsView.addItem(lim)

        self.add_sets()

    def __call__(self):
        self.clear()
        self.add_sets()
        # s.valuesChanged.connect(self.management.shift_scale)

    def clear(self):
        self.dsc = []
        self.listWidget.clear()
        for v in self._plots.values():
            self.graphicsView.removeItem(v)
        self._plots = {}

        for v in self._tg_baselines.values():
            self.graphicsView.removeItem(v)
        self._tg_baselines = {}

    def add_sets(self):
        for key, name in self._management.active_sets:
            data = self._management.data[key]
            if isinstance(data.data, DSC):
                self.dsc.append(data)
            else:
                continue

            item = QtWidgets.QListWidgetItem(name)
            item.setData(QtCore.Qt.UserRole, key)
            self.listWidget.addItem(item)

            plot = PlotItem(x=data.x, y=data.y)
            self.graphicsView.addItem(plot)
            self._plots[key] = plot

    @QtCore.pyqtSlot(name='on_pushButton_clicked')
    def calc_tg(self):
        baselines = tuple(lim.getRegion() for lim in self.limits)
        if baselines[0][0] > baselines[1][0]:
            baselines = baselines[1], baselines[0]

        for idx in range(self.listWidget.count()):
            key = self.listWidget.item(idx).data(QtCore.Qt.UserRole)
            tg_results, glass, liquid, tangent = self._management[key].data.glass_transition(*baselines)

            lines = []
            for line in (glass, liquid, tangent):
                plt = PlotItem(line[:, 0], line[:, 1])
                self.graphicsView.addItem(plt)
                lines.append(plt)
            self._tg_baselines[key] = lines

            tg_plot = PlotItem(*zip((i for i in tg_results.values())))
            self.graphicsView.addItem(tg_plot)

    def close(self) -> bool:
        self.clear()

        return super().close()

